"""Container to stack elements with spacing."""

from pynecone.components.libs.chakra import ChakraComponent
from pynecone.components.tags import Tag
from pynecone.var import Var


class Fade(ChakraComponent):
    """Display a square box."""

    tag = "Fade"


class ScaleFade(ChakraComponent):
    """Display a square box."""

    tag = "ScaleFade"


class Slide(ChakraComponent):
    """Display a square box."""

    tag = "Slide"


class SlideFade(ChakraComponent):
    """Display a square box."""

    tag = "SlideFade"



"""Typography components."""

from pynecone.components.component import Component

from .heading import Heading
from .markdown import Markdown
from .text import Text


def span(text: str, **kwargs) -> Component:
    """Create a span component.

    Args:
        text: The text to display.
        **kwargs: The keyword arguments to pass to the Text component.

    Returns:
        The span component.
    """
    return Text.create(text, as_="span", **kwargs)


__all__ = [f for f in dir() if f[0].isupper() or f in ("span",)]  # type: ignore



"""Common utility functions used in the compiler."""

from __future__ import annotations

import json
from typing import TYPE_CHECKING, Dict, Set, Type

from pynecone import constants, utils
from pynecone.compiler import templates
from pynecone.components.base import (
    Body,
    DocumentHead,
    Head,
    Html,
    Link,
    Main,
    Script,
    Title,
)
from pynecone.components.component import ImportDict
from pynecone.state import State
from pynecone.style import Style

if TYPE_CHECKING:
    from pynecone.components.component import Component


# To re-export this function.
merge_imports = utils.merge_imports


def compile_import_statement(lib: str, fields: Set[str]) -> str:
    """Compile an import statement.

    Args:
        lib: The library to import from.
        fields: The set of fields to import from the library.

    Returns:
        The compiled import statement.
    """
    # Check for default imports.
    defaults = {
        field
        for field in fields
        if field.lower() == lib.lower().replace("-", "").replace("/", "")
    }
    assert len(defaults) < 2

    # Get the default import, and the specific imports.
    default = next(iter(defaults), "")
    rest = fields - defaults
    return templates.format_import(lib=lib, default=default, rest=rest)


def compile_imports(imports: ImportDict) -> str:
    """Compile an import dict.

    Args:
        imports: The import dict to compile.

    Returns:
        The compiled import dict.
    """
    return templates.join(
        [compile_import_statement(lib, fields) for lib, fields in imports.items()]
    )


def compile_constant_declaration(name: str, value: str) -> str:
    """Compile a constant declaration.

    Args:
        name: The name of the constant.
        value: The value of the constant.

    Returns:
        The compiled constant declaration.
    """
    return templates.CONST(name=name, value=json.dumps(value))


def compile_constants() -> str:
    """Compile all the necessary constants.

    Returns:
        A string of all the compiled constants.
    """
    endpoint = constants.Endpoint.EVENT
    return templates.join(
        [compile_constant_declaration(name=endpoint.name, value=endpoint.get_url())]
    )


import plotly.graph_objects as go


def compile_state(state: Type[State]) -> str:
    """Compile the state of the app.

    Args:
        state: The app state object.

    Returns:
        A string of the compiled state.
    """
    initial_state = state().dict()
    initial_state.update(
        {
            "events": [{"name": utils.get_hydrate_event(state)}],
        }
    )
    initial_state = utils.format_state(initial_state)
    synced_state = templates.format_state(
        state=state.get_name(), initial_state=json.dumps(initial_state)
    )
    initial_result = {
        constants.STATE: None,
        constants.EVENTS: [],
        constants.PROCESSING: False,
    }
    result = templates.format_state(
        state="result",
        initial_state=json.dumps(initial_result),
    )
    router = templates.ROUTER
    return templates.join([synced_state, result, router])


def compile_events(state: Type[State]) -> str:
    """Compile all the events for a given component.

    Args:
        state: The state class for the component.

    Returns:
        A string of the compiled events for the component.
    """
    state_name = state.get_name()
    state_setter = templates.format_state_setter(state_name)
    return templates.EVENT_FN(state=state_name, set_state=state_setter)


def compile_effects(state: Type[State]) -> str:
    """Compile all the effects for a given component.

    Args:
        state: The state class for the component.

    Returns:
        A string of the compiled effects for the component.
    """
    state_name = state.get_name()
    set_state = templates.format_state_setter(state_name)
    return templates.USE_EFFECT(state=state_name, set_state=set_state)


def compile_render(component: Component) -> str:
    """Compile the component's render method.

    Args:
        component: The component to compile the render method for.

    Returns:
        A string of the compiled render method.
    """
    return component.render()


def create_document_root(stylesheets) -> Component:
    """Create the document root.

    Args:
        stylesheets: The stylesheets to include in the document root.

    Returns:
        The document root.
    """
    sheets = [Link.create(rel="stylesheet", href=href) for href in stylesheets]
    return Html.create(
        DocumentHead.create(*sheets),
        Body.create(
            Main.create(),
            Script.create(),
        ),
    )


def create_theme(style: Style) -> Dict:
    """Create the base style for the app.

    Args:
        style: The style dict for the app.

    Returns:
        The base style for the app.
    """
    return {
        "styles": {
            "global": Style({k: v for k, v in style.items() if not isinstance(k, type)})
        },
    }


def add_title(page: Component, title: str) -> Component:
    """Add a title to a page.

    Args:
        page: The component for the page.
        title: The title to add.

    Returns:
        The component with the title added.
    """
    page.children.append(Head.create(Title.create(title)))
    return page



"""A box that centers its contents."""

from pynecone.components.libs.chakra import ChakraComponent


class Center(ChakraComponent):
    """Center is a layout component that centers its child within itself. It's useful for centering text, images, and other elements. All box can be used on center to style."""

    tag = "Center"


class Square(ChakraComponent):
    """Square centers its child given size (width and height). All box props can be used on Square."""

    tag = "Square"


class Circle(ChakraComponent):
    """Circle a Square with round border-radius. All box props can be used on Circle."""

    tag = "Circle"



"""A text component."""
from __future__ import annotations

from pynecone.components.libs.chakra import ChakraComponent
from pynecone.components.tags import Tag
from pynecone.var import Var


class Text(ChakraComponent):
    """Text component is the used to render text and paragraphs within an interface. It renders a p tag by default."""

    tag = "Text"

    # The text to display.
    text: Var[str]

    # The CSS `text-align` property
    text_align: Var[str]

    # The CSS `text-transform` property
    casing: Var[str]

    # The CSS `text-decoration` property
    decoration: Var[str]

    # Override the text element. Tpyes are b, strong, i, em, mark, small, del, ins, sub, and sup.
    as_: Var[str]



"""The Pynecone config."""

from typing import Optional

from pynecone import constants
from pynecone.base import Base


class Config(Base):
    """A Pynecone config."""

    # The name of the app.
    app_name: str

    # The username.
    username: Optional[str] = None

    # The backend API url.
    api_url: str = constants.API_URL

    # The database url.
    db_url: str = constants.DB_URL

    # The redis url.
    redis_url: Optional[str] = None

    # The deploy url.
    deploy_url: Optional[str] = None

    # The environment mode.
    env: constants.Env = constants.Env.DEV

    # The path to the bun executable.
    bun_path: str = constants.BUN_PATH



"""List components."""

from pynecone.components.component import Component
from pynecone.components.libs.chakra import ChakraComponent
from pynecone.var import Var


class List(ChakraComponent):
    """List component is used to display list items. It renders a ul element by default."""

    tag = "List"

    # The space between each list item
    spacing: Var[str]

    # Shorthand prop for listStylePosition
    style_position: Var[str]

    # Shorthand prop for listStyleType
    style_type: Var[str]


class ListItem(ChakraComponent):
    """ListItem composes Box so you can pass all style and pseudo style props."""

    tag = "ListItem"


class OrderedList(ChakraComponent):
    """An ordered list component."""

    tag = "OrderedList"


class UnorderedList(ChakraComponent):
    """An unordered list component."""

    tag = "UnorderedList"



"""Container to observer element."""

from pynecone.components.component import Component


class Observer(Component):
    """A component that wraps a react-visibility-sensor component."""

    library = "react-visibility-sensor"


class VisibilitySensor(Observer):
    """Display a square box."""

    tag = "VisibilitySensor"



"""The main Pynecone app."""

import os
import re
from typing import Any, Callable, Coroutine, Dict, List, Optional, Tuple, Type, Union

import fastapi
from fastapi.middleware import cors

from pynecone import constants, utils
from pynecone.base import Base
from pynecone.compiler import compiler
from pynecone.compiler import utils as compiler_utils
from pynecone.components.component import Component, ComponentStyle
from pynecone.event import Event
from pynecone.middleware import HydrateMiddleware, Middleware
from pynecone.model import Model
from pynecone.state import DefaultState, Delta, State, StateManager, StateUpdate

# Define custom types.
ComponentCallable = Callable[[], Component]
Reducer = Callable[[Event], Coroutine[Any, Any, StateUpdate]]


class App(Base):
    """A Pynecone application."""

    # A map from a page route to the component to render.
    pages: Dict[str, Component] = {}

    # A list of URLs to stylesheets to include in the app.
    stylesheets: List[str] = []

    # The backend API object.
    api: fastapi.FastAPI = None  # type: ignore

    # The state class to use for the app.
    state: Type[State] = DefaultState

    # Class to manage many client states.
    state_manager: StateManager = StateManager()

    # The styling to apply to each component.
    style: ComponentStyle = {}

    # Middleware to add to the app.
    middleware: List[Middleware] = []

    def __init__(self, *args, **kwargs):
        """Initialize the app.

        Args:
            *args: Args to initialize the app with.
            **kwargs: Kwargs to initialize the app with.
        """
        super().__init__(*args, **kwargs)

        # Add middleware.
        self.middleware.append(HydrateMiddleware())

        # Set up the state manager.
        self.state_manager.setup(state=self.state)

        # Set up the API.
        self.api = fastapi.FastAPI()
        self.add_cors()
        self.add_default_endpoints()

    def __repr__(self) -> str:
        """Get the string representation of the app.

        Returns:
            The string representation of the app.
        """
        return f"<App state={self.state.__name__}>"

    def add_default_endpoints(self):
        """Add the default endpoints."""
        # To test the server.
        self.get(str(constants.Endpoint.PING))(_ping)

        # To make state changes.
        self.post(str(constants.Endpoint.EVENT))(_event(app=self))

    def add_cors(self):
        """Add CORS middleware to the app."""
        self.api.add_middleware(
            cors.CORSMiddleware,
            allow_origins=["*"],
            allow_methods=["*"],
            allow_headers=["*"],
        )

    def get(self, path: str, *args, **kwargs) -> Callable:
        """Register a get request.

        Args:
            path: The endpoint path to link to the request.
            *args: Args to pass to the request.
            **kwargs: Kwargs to pass to the request.

        Returns:
            A decorator to handle the request.
        """
        return self.api.get(path, *args, **kwargs)

    def post(self, path: str, *args, **kwargs) -> Callable:
        """Register a post request.

        Args:
            path: The endpoint path to link to the request.
            *args: Args to pass to the request.
            **kwargs: Kwargs to pass to the request.

        Returns:
            A decorator to handle the request.
        """
        return self.api.post(path, *args, **kwargs)

    def preprocess(self, state: State, event: Event) -> Optional[Delta]:
        """Preprocess the event.

        This is where middleware can modify the event before it is processed.
        Each middleware is called in the order it was added to the app.

        If a middleware returns a delta, the event is not processed and the
        delta is returned.

        Args:
            state: The state to preprocess.
            event: The event to preprocess.

        Returns:
            An optional state to return.
        """
        for middleware in self.middleware:
            out = middleware.preprocess(app=self, state=state, event=event)
            if out is not None:
                return out

    def postprocess(self, state: State, event: Event, delta: Delta) -> Optional[Delta]:
        """Postprocess the event.

        This is where middleware can modify the delta after it is processed.
        Each middleware is called in the order it was added to the app.

        If a middleware returns a delta, the delta is not processed and the
        delta is returned.

        Args:
            state: The state to postprocess.
            event: The event to postprocess.
            delta: The delta to postprocess.

        Returns:
            An optional state to return.
        """
        for middleware in self.middleware:
            out = middleware.postprocess(
                app=self, state=state, event=event, delta=delta
            )
            if out is not None:
                return out

    def add_page(
        self,
        component: Union[Component, ComponentCallable],
        path: Optional[str] = None,
        title: str = constants.DEFAULT_TITLE,
    ):
        """Add a page to the app.

        If the component is a callable, by default the route is the name of the
        function. Otherwise, a route must be provided.

        Args:
            component: The component to display at the page.
            path: The path to display the component at.
            title: The title of the page.
        """
        # If the path is not set, get it from the callable.
        if path is None:
            assert isinstance(
                component, Callable
            ), "Path must be set if component is not a callable."
            path = component.__name__

        from pynecone.var import BaseVar

        parts = os.path.split(path)
        check = re.compile(r"^\[(.+)\]$")
        args = []
        for part in parts:
            match = check.match(part)
            if match:
                v = BaseVar(
                    name=match.groups()[0],
                    type_=str,
                    state=f"{constants.ROUTER}.query",
                )
                args.append(v)

        # Generate the component if it is a callable.
        component = component if isinstance(component, Component) else component(*args)

        # Add the title to the component.
        compiler_utils.add_title(component, title)

        # Format the route.
        route = utils.format_route(path)

        # Add the page.
        self.pages[route] = component

    def compile(self, force_compile: bool = False):
        """Compile the app and output it to the pages folder.

        If the config environment is set to production, the app will
        not be compiled.

        Args:
            force_compile: Whether to force the app to compile.
        """
        # Get the env mode.
        config = utils.get_config()
        if config.env != constants.Env.DEV and not force_compile:
            print("Skipping compilation in non-dev mode.")
            return

        # Create the database models.
        Model.create_all()

        # Create the root document with base styles and fonts.
        self.pages[constants.DOCUMENT_ROOT] = compiler_utils.create_document_root(
            self.stylesheets
        )
        self.pages[constants.THEME] = compiler_utils.create_theme(self.style)  # type: ignore

        # Compile the pages.
        for path, component in self.pages.items():
            self.compile_page(path, component)

    def compile_page(
        self, path: str, component: Component, write: bool = True
    ) -> Tuple[str, str]:
        """Compile a single page.

        Args:
            path: The path to compile the page to.
            component: The component to compile.
            write: Whether to write the page to the pages folder.

        Returns:
            The path and code of the compiled page.
        """
        # Get the path for the output file.
        output_path = utils.get_page_path(path)

        # Compile the document root.
        if path == constants.DOCUMENT_ROOT:
            code = compiler.compile_document_root(component)

        # Compile the theme.
        elif path == constants.THEME:
            output_path = utils.get_theme_path()
            code = compiler.compile_theme(component)  # type: ignore

        # Compile all other pages.
        else:
            # Add the style to the component.
            component.add_style(self.style)
            code = compiler.compile_component(
                component=component,
                state=self.state,
            )

        # Write the page to the pages folder.
        if write:
            utils.write_page(output_path, code)

        return output_path, code

    def get_state(self, token: str) -> State:
        """Get the state for a token.

        Args:
            token: The token to get the state for.

        Returns:
            The state for the token.
        """
        return self.state_manager.get_state(token)

    def set_state(self, token: str, state: State):
        """Set the state for a token.

        Args:
            token: The token to set the state for.
            state: The state to set.
        """
        self.state_manager.set_state(token, state)


async def _ping() -> str:
    """Test API endpoint.

    Returns:
        The response.
    """
    return "pong"


def _event(app: App) -> Reducer:
    """Create an event reducer to modify the state.

    Args:
        app: The app to modify the state of.

    Returns:
        A handler that takes in an event and modifies the state.
    """

    async def process(event: Event) -> StateUpdate:
        # Get the state for the session.
        state = app.get_state(event.token)

        # Preprocess the event.
        pre = app.preprocess(state, event)
        if pre is not None:
            return StateUpdate(delta=pre)

        # Apply the event to the state.
        update = await state.process(event)
        app.set_state(event.token, state)

        # Postprocess the event.
        post = app.postprocess(state, event, update.delta)
        if post is not None:
            return StateUpdate(delta=post)

        # Return the delta.
        return update

    return process



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pynecone',
 'pynecone..templates.app',
 'pynecone.compiler',
 'pynecone.components',
 'pynecone.components.base',
 'pynecone.components.datadisplay',
 'pynecone.components.disclosure',
 'pynecone.components.feedback',
 'pynecone.components.forms',
 'pynecone.components.graphing',
 'pynecone.components.layout',
 'pynecone.components.libs',
 'pynecone.components.media',
 'pynecone.components.navigation',
 'pynecone.components.overlay',
 'pynecone.components.tags',
 'pynecone.components.transitions',
 'pynecone.components.typography',
 'pynecone.middleware']

package_data = \
{'': ['*'],
 'pynecone': ['.templates/assets/*',
              '.templates/web/.gitignore',
              '.templates/web/.gitignore',
              '.templates/web/.gitignore',
              '.templates/web/next.config.js',
              '.templates/web/next.config.js',
              '.templates/web/next.config.js',
              '.templates/web/package.json',
              '.templates/web/package.json',
              '.templates/web/package.json',
              '.templates/web/pages/*',
              '.templates/web/utils/*']}

install_requires = \
['fastapi>=0.88.0,<0.89.0',
 'gunicorn>=20.1.0,<21.0.0',
 'plotly>=5.10.0,<6.0.0',
 'pydantic==1.10.2',
 'redis>=4.3.5,<5.0.0',
 'requests>=2.28.1,<3.0.0',
 'rich>=12.6.0,<13.0.0',
 'sqlmodel>=0.0.8,<0.0.9',
 'typer>=0.7.0,<0.8.0',
 'uvicorn>=0.20.0,<0.21.0']

entry_points = \
{'console_scripts': ['pc = pynecone.pc:main']}

setup_kwargs = {
    'name': 'pynecone-io',
    'version': '0.1.6',
    'description': 'The easiest way to build web apps.',
    'long_description': '<div align="center">\n\n<img src="docs/images/logo.png" alt="drawing" width = 450/>\n\n**The easiest way to build and deploy web apps.**\n\n[![PyPI version](https://badge.fury.io/py/pynecone-io.svg)](https://badge.fury.io/py/pynecone-io)\n![versions](https://img.shields.io/pypi/pyversions/pynecone-io.svg)\n[![License](https://img.shields.io/badge/License-Apache_2.0-yellowgreen.svg)](https://opensource.org/licenses/Apache-2.0)  \n\n\n<div align="left">\n\n## Getting Started\n\nPynecone is a full-stack python framework that makes it easy to build and deploy web apps in minutes.\n\nAll the information for getting started can be found in this README, however, a more detailed explanation of the following topics can be found on our website:\n\n<div align="center">\n\n### [Introduction](https://pynecone.io/docs/getting-started/introduction) | [Component Library](https://pynecone.io/docs/library) | [Examples](https://pynecone.io) | [Deployment](https://pynecone.io/docs/hosting/deploy) \n\n<div align="left">\n\n## Installation\nPynecone requires to following to get started:\n* Python 3.7+\n* [NodeJS 12.22.0+](https://nodejs.org/en/)\n\n```\n$ pip install pynecone-io\n```\n\n## Create your first Pynecone app\n\nInstalling Pynecone also installs the pc command line tool. Test that the install was successful by creating a new project. \n\nReplace my_app_name with your project name:\n\n```\n$ mkdir my_app_name\n$ cd my_app_name\n$ pc init\n```\n\nThis initializes a template app in your new directory.\nYou can run this app in development mode:\n```\n$ pc run\n```\n\n\nYou should see your app running at http://localhost:3000.\n\n\nNote that the port may be different if you have another app running on port 3000.\n\n\nNow you can modify the source code in my_app_name/my_app_name.py. Pynecone has fast refreshes so you can see your changes instantly when you save your code.\n\n## Example App\n\nLet\'s go over a simple counter app to explore the basics of Pynecone.\n\n<div align="center">\n<img src="docs/images/counter.gif" alt="drawing" width="550"/>\n<div align="left">\n\nHere is the complete code to create this.\n\n```python\nimport pynecone as pc\nimport random\n\nclass State(pc.State):\n    count = 0\n\n    def increment(self):\n        """Increment the count."""\n        self.count += 1\n\n    def decrement(self):\n        """Decrement the count."""\n        self.count -= 1\n\n    def random(self):\n        """Randomize the count."""\n        self.count = random.randint(0, 100)\n\n\ndef index():\n    """The main view."""\n    return pc.center(\n        pc.vstack(\n            pc.heading(State.count),\n            pc.hstack(\n                pc.button("Decrement", on_click=State.decrement, color_scheme="red"),\n                pc.button(\n                    "Randomize",\n                    on_click=State.random,\n                    background_image="linear-gradient(90deg, rgba(255,0,0,1) 0%, rgba(0,176,34,1) 100%)",\n                    color="white",\n                ),\n                pc.button("Increment", on_click=State.increment, color_scheme="green"),\n            ),\n            padding="1em",\n            bg="#ededed",\n            border_radius="1em",\n            box_shadow="lg",\n        ),\n        padding_y="5em",\n        font_size="2em",\n        text_align="center",\n    )\n\n\n# Add state and page to the app.\napp = pc.App(state=State)\napp.add_page(index, title="Counter")\napp.compile()\n```\nLet\'s break this down.\n\n* ### State\n    \n``` python\nclass State(pc.State):\n    count = 0 \n```\nThe state defines all the variables (called vars) in an app that can change, as well as the functions that change them.\nHere our state has by a single var, count, which holds the current value of the counter.\nThe frontend of the app is a reflection of the current state.\n\n    \n* ### Event Handlers\n```python\ndef increment(self):\n    """Increment the count."""\n    self.count += 1\n\ndef decrement(self):\n    """Decrement the count."""\n    self.count -= 1\n\ndef random(self):\n    """Randomize the count."""\n    self.count = random.randint(0, 100)\n```\nWithin the state, we define functions, called event handlers, that change the state vars.\nEvent handlers are the only way that we can modify the state in Pynecone. They can be called in response to user actions, such as clicking a button or typing in a text box. These actions are called events.\nOur counter app has two event handlers, increment and decrement.\n    \n* ### Frontend\n\n```python \ndef index():\n    return pc.center(\n        pc.vstack(\n            pc.heading(State.count),\n            pc.hstack(\n                pc.button("Decrement", on_click=State.decrement, color_scheme="red"),\n                pc.button(\n                    "Randomize",\n                    on_click=State.random,\n                    background_image="linear-gradient(90deg, rgba(255,0,0,1) 0%, rgba(0,176,34,1) 100%)",\n                    color="white",\n                ),\n                pc.button("Increment", on_click=State.increment, color_scheme="green"),\n            ),\n            padding="1em",\n            bg="#ededed",\n            border_radius="1em",\n            box_shadow="lg",\n        ),\n        padding_y="5em",\n        font_size="2em",\n        text_align="center",\n    )\n```\nThis function defines the frontend of the app.\nWe use different components such as pc.box, pc.button, and pc.heading to build the frontend. Components can be nested to create complex layouts, and can be styled using the full power of CSS.\n    \nPynecone comes with [50+ built-in components](https://pynecone.io/docs/library) to help you get started. \nWe are actively adding more components, plus it\'s easy to create your own components.\n\n* ### Routing \n    \nNext we define our app and tell it what state to use.\n```python\napp = pc.App(state=State)\n```\nWe add a route from the root of the app to the counter component. By default the route\n```python\napp.add_page(index)\n```\nYou can create a multi-page app by adding more routes.\n    \n## Contributing\n\nPull requests are encouraged and always welcome. Pick an issue and help us out, or submit an issue id something is not working or confusing!\n\n## More Information \n\nMore information about Pynecone can be found on our website, https://pynecone.io\n  \n## License\n\nPynecone is open-source and licensed under the [Apache License 2.0](LICENSE)\n',
    'author': 'Nikhil Rao',
    'author_email': 'nikhil@pynecone.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pynecone.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)



"""Constants used throughout the package."""

import os
from enum import Enum

import pkg_resources

# App names and versions.
# The name of the Pynecone module.
MODULE_NAME = "pynecone"
# The name of the pip install package.
PACKAGE_NAME = "pynecone-io"
# The current version of Pynecone.
VERSION = pkg_resources.get_distribution(PACKAGE_NAME).version

# Files and directories used to init a new project.
# The root directory of the pynecone library.
ROOT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
# The name of the file used for pc init.
APP_TEMPLATE_FILE = "tutorial.py"
# The name of the assets directory.
APP_ASSETS_DIR = "assets"
# The template directory used during pc init.
TEMPLATE_DIR = os.path.join(ROOT_DIR, MODULE_NAME, ".templates")
# The web subdirectory of the template directory.
WEB_TEMPLATE_DIR = os.path.join(TEMPLATE_DIR, "web")
# The app subdirectory of the template directory.
APP_TEMPLATE_DIR = os.path.join(TEMPLATE_DIR, "app")
# The assets subdirectory of the template directory.
ASSETS_TEMPLATE_DIR = os.path.join(TEMPLATE_DIR, APP_ASSETS_DIR)

# The frontend directories in a project.
# The web folder where the NextJS app is compiled to.
WEB_DIR = ".web"
# The name of the utils file.
UTILS_DIR = "utils"
# The name of the state file.
STATE_PATH = os.path.join(UTILS_DIR, "state")
# The directory where the app pages are compiled to.
WEB_PAGES_DIR = os.path.join(WEB_DIR, "pages")
# The directory where the utils file is located.
WEB_UTILS_DIR = os.path.join(WEB_DIR, UTILS_DIR)
# The directory where the assets are located.
WEB_ASSETS_DIR = os.path.join(WEB_DIR, "public")
# The node modules directory.
NODE_MODULES = "node_modules"
# The package lock file.
PACKAGE_LOCK = "package-lock.json"

# Commands to run the app.
# The backend api url.
API_URL = "http://localhost:8000"
# The default path where bun is installed.
BUN_PATH = "$HOME/.bun/bin/bun"
# Command to install bun.
INSTALL_BUN = "curl https://bun.sh/install | bash"
# Command to run the backend in dev mode.
RUN_BACKEND = "uvicorn --log-level critical --reload --host 0.0.0.0".split()
# The number of workers to run in production mode by default.
NUM_WORKERS = (os.cpu_count() or 1) * 2 + 1
# The default timeout when launching the gunicorn server.
TIMEOUT = 120
# The command to run the backend in production mode.
RUN_BACKEND_PROD = f"gunicorn --worker-class uvicorn.workers.UvicornH11Worker --bind 0.0.0.0:8000 --workers {NUM_WORKERS} --threads {NUM_WORKERS} --preload --timeout {TIMEOUT} --log-level debug".split()

# Compiler variables.
# The extension for compiled Javascript files.
JS_EXT = ".js"
# The extension for python files.
PY_EXT = ".py"
# The expected variable name where the pc.App is stored.
APP_VAR = "app"
# The expected variable name where the API object is stored for deployment.
API_VAR = "api"
# The name of the router variable.
ROUTER = "router"
# The name of the variable to hold API results.
RESULT = "result"
# The name of the process variable.
PROCESSING = "processing"
# The name of the state variable.
STATE = "state"
# The name of the events variable.
EVENTS = "events"
# The name of the initial hydrate event.
HYDRATE = "hydrate"
# The name of the index page.
INDEX_ROUTE = "index"
# The name of the document root page.
DOCUMENT_ROOT = "_document"
# The name of the theme page.
THEME = "theme"
# The prefix used to create setters for state vars.
SETTER_PREFIX = "set_"
# The name of the frontend zip during deployment.
FRONTEND_ZIP = "frontend.zip"
# The name of the backend zip during deployment.
BACKEND_ZIP = "backend.zip"
# The name of the sqlite database.
DB_NAME = "pynecone.db"
# The sqlite url.
DB_URL = f"sqlite:///{DB_NAME}"
# The default title to show for Pynecone apps.
DEFAULT_TITLE = "Pynecone App"
# The name of the pynecone config module.
CONFIG_MODULE = "pcconfig"
# The python config file.
CONFIG_FILE = f"{CONFIG_MODULE}{PY_EXT}"
# The deployment URL.
PRODUCTION_BACKEND_URL = "https://{username}-{app_name}.api.pynecone.app"
# Token expiration time in seconds.
TOKEN_EXPIRATION = 60 * 60


# Env modes
class Env(str, Enum):
    """The environment modes."""

    DEV = "dev"
    PROD = "prod"


class Endpoint(Enum):
    """Endpoints for the pynecone backend API."""

    PING = "ping"
    EVENT = "event"

    def __str__(self) -> str:
        """Get the string representation of the endpoint.

        Returns:
            The path for the endpoint.
        """
        return f"/{self.value}"

    def get_url(self) -> str:
        """Get the URL for the endpoint.

        Returns:
            The full URL for the endpoint.
        """
        from pynecone import utils

        config = utils.get_config()
        return "".join([config.api_url, str(self)])
